@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001,2002,2003,2004  Aymeric MOIZARD jack@atosc.org
+  Copyright (C) 2001,2002,2003,2004,2005,2006,2007 Aymeric MOIZARD jack@atosc.org
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -30,11 +30,12 @@
 
 #ifndef DOXYGEN
 
-#if !defined(__PALMOS__) && (defined(WIN32) || defined(_WIN32_WCE))
+#if !((defined(__PALMOS__) || defined(HAVE_STRUCT_TIMEVAL)))
 /* Struct timeval */
-struct timeval {
-        long    tv_sec;         /* seconds */
-        long    tv_usec;        /* and microseconds */
+struct timeval
+{
+  long tv_sec;                  /* seconds */
+  long tv_usec;                 /* and microseconds */
 };
 #endif
 
@@ -66,6 +67,7 @@ struct __payload
  * @param payload The payload.
  */
 int __payload_init (__payload_t ** payload);
+
 /**
  * Free a payload element.
  * @param payload The payload.
@@ -90,6 +92,11 @@ void __payload_free (__payload_t * payload);
 /* - Win32: Pthreads for Win32 (http://sources.redhat.com/pthreads-win32). */
 #if defined(HAVE_PTHREAD) || defined(HAVE_PTHREAD_H) || defined(HAVE_PTH_PTHREAD_H) || \
 	defined(HAVE_PTHREAD_WIN32)
+#if defined(__arc__)
+#include <ucos_ii_api.h>
+#endif
+
+
 #include <pthread.h>
 typedef pthread_t osip_thread_t;
 #endif
@@ -132,7 +139,7 @@ osip_thread_t;
 /* Is there any semaphore implementation available? */
 #if !defined(HAVE_SEMAPHORE_H) && !defined(HAVE_SYS_SEM_H) && \
     !defined(WIN32) && !defined(_WIN32_WCE) && !defined(HAVE_PTHREAD_WIN32) && \
-    !defined(__PSOS__) && !defined(__VXWORKS_OS__)
+    !defined(__PSOS__) && !defined(__VXWORKS_OS__) && !defined(__arc__)
 #error No semaphore implementation found
 #endif
 
@@ -148,7 +155,17 @@ typedef pthread_mutex_t osip_mutex_t;
 #include <synch.h>
 #endif
 
-#if (defined(HAVE_SEMAPHORE_H) && !defined(__APPLE_CC__)) || defined(HAVE_PTHREAD_WIN32)
+#if defined(__arc__)
+
+typedef struct
+{
+  int _sem_counter;
+  struct osip_mutex *_sem_mutex;
+} sem_t;
+
+typedef sem_t osip_sem_t;
+
+#elif (defined(HAVE_SEMAPHORE_H) && !defined(__APPLE_CC__)) || defined(HAVE_PTHREAD_WIN32)
 #include <semaphore.h>
 #ifdef __sun__
 #undef getdate
@@ -177,11 +194,24 @@ osip_sem_t;
 #define _WINSOCKAPI_
 #include <windows.h>
 #undef _WINSOCKAPI_
+
+#if (_WIN32_WINNT >= 0x0500)
+
+#define OSIP_CRITICALSECTION_SPIN  4000
+typedef struct
+{
+  CRITICAL_SECTION h;
+}
+osip_mutex_t;
+#else
+
 typedef struct
 {
   HANDLE h;
 }
 osip_mutex_t;
+#endif
+
 typedef struct
 {
   HANDLE h;
