@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001,2002,2003,2004  Aymeric MOIZARD jack@atosc.org
+  Copyright (C) 2001,2002,2003,2004,2005,2006,2007 Aymeric MOIZARD jack@atosc.org
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -50,7 +50,7 @@ extern "C"
  * a copy of ip_addr is done.
  */
   int osip_message_fix_last_via_header (osip_message_t * request,
-					const char *ip_addr, int port);
+                                        const char *ip_addr, int port);
 
 
 /**
@@ -82,102 +82,153 @@ extern "C"
  * @param sip The element to work on.
  * @param hvalue The string describing the element.
  */
+#ifndef MINISIZE
   int osip_message_set_accept (osip_message_t * sip, const char *hvalue);
+#else
+#define osip_message_set_accept(sip,value)            osip_message_set_header((osip_message_t *)sip,(const char *)"Accept",value)
+#endif
 /**
  * Get one Accept header.
  * @param sip The element to work on.
  * @param pos The index of the element to get.
  * @param dest A pointer on the header found.
  */
+#ifndef MINISIZE
   int osip_message_get_accept (const osip_message_t * sip, int pos,
-			       osip_accept_t ** dest);
+                               osip_accept_t ** dest);
+#else
+#define osip_message_get_accept(sip,pos,dest)          osip_message_header_get_byname(( osip_message_t *)sip,(const char *)"accept",pos,(osip_header_t **)dest)
+#endif
 /**
  * Set the Accept-encoding header.
  * @param sip The element to work on.
  * @param hvalue The string describing the element.
  */
-  int osip_message_set_accept_encoding (osip_message_t * sip,
-					const char *hvalue);
+#ifndef MINISIZE
+  int osip_message_set_accept_encoding (osip_message_t * sip, const char *hvalue);
+#else
+#define osip_message_set_accept_encoding(sip,value)            osip_message_set_header((osip_message_t *)sip,(const char *)"Accept-Encoding",value)
+#endif
 /**
  * Get one Accept-encoding header.
  * @param sip The element to work on.
  * @param pos The index of the element to get.
  * @param dest A pointer on the header found.
  */
+#ifndef MINISIZE
   int osip_message_get_accept_encoding (const osip_message_t * sip, int pos,
-					osip_accept_encoding_t ** dest);
+                                        osip_accept_encoding_t ** dest);
+#else
+#define osip_message_get_accept_encoding(sip,pos,dest)          osip_message_header_get_byname(( osip_message_t *)sip,(const char *)"accept-encoding",pos,(osip_header_t **)dest)
+#endif
 /**
  * Set the Accept-language header.
  * @param sip The element to work on.
  * @param hvalue The string describing the element.
  */
-  int osip_message_set_accept_language (osip_message_t * sip,
-					const char *hvalue);
+#ifndef MINISIZE
+  int osip_message_set_accept_language (osip_message_t * sip, const char *hvalue);
+#else
+#define osip_message_set_accept_language(sip,value)            osip_message_set_header((osip_message_t *)sip,(const char *)"Accept-Language",value)
+#endif
 /**
- * Get one Accept header.
+ * Get one Accept-Language header.
  * @param sip The element to work on.
  * @param pos The index of the element to get.
  * @param dest A pointer on the header found.
  */
+#ifndef MINISIZE
   int osip_message_get_accept_language (const osip_message_t * sip, int pos,
-					osip_accept_language_t ** dest);
+                                        osip_accept_language_t ** dest);
+#else
+#define osip_message_get_accept_language(sip,pos,dest)          osip_message_header_get_byname(( osip_message_t *)sip,(const char *)"accept-language",pos,(osip_header_t **)dest)
+#endif
 /**
  * Set the Alert-info header.
  * @param sip The element to work on.
  * @param hvalue The string describing the element.
  */
+#ifndef MINISIZE
   int osip_message_set_alert_info (osip_message_t * sip, const char *hvalue);
+#else
+#define osip_message_set_alert_info(sip,value)            osip_message_set_header((osip_message_t *)sip,(const char *)"Alert-Info",value)
+#endif
 /**
  * Get one Alert-info header.
  * @param sip The element to work on.
  * @param pos The index of the element to get.
  * @param dest A pointer on the header found.
  */
+#ifndef MINISIZE
   int osip_message_get_alert_info (const osip_message_t * sip, int pos,
-				   osip_alert_info_t ** dest);
+                                   osip_alert_info_t ** dest);
+#else
+#define osip_message_get_alert_info(sip,pos,dest)          osip_message_header_get_byname(( osip_message_t *)sip,(const char *)"alert-info",pos,(osip_header_t **)dest)
+#endif
 /**
  * Set the Allow header.
  * @param sip The element to work on.
  * @param hvalue The string describing the element.
  */
+#ifndef MINISIZE
   int osip_message_set_allow (osip_message_t * sip, const char *hvalue);
+#else
+#define osip_message_set_allow(sip,value)            osip_message_set_header((osip_message_t *)sip,(const char *)"Allow",value)
+#endif
 /**
  * Get one Allow header.
  * @param sip The element to work on.
  * @param pos The index of the element to get.
  * @param dest A pointer on the header found.
  */
+#ifndef MINISIZE
   int osip_message_get_allow (const osip_message_t * sip, int pos,
-			      osip_allow_t ** dest);
+                              osip_allow_t ** dest);
+#else
+#define osip_message_get_allow(sip,pos,dest)          osip_message_header_get_byname(( osip_message_t *)sip,(const char *)"allow",pos,(osip_header_t **)dest)
+#endif
 /**
  * Set the Authentication-info header.
  * @param sip The element to work on.
  * @param hvalue The string describing the element.
  */
-  int osip_message_set_authentication_info (osip_message_t * sip, const char *hvalue);
+#ifndef MINISIZE
+  int osip_message_set_authentication_info (osip_message_t * sip,
+                                            const char *hvalue);
+#else
+#define osip_message_set_authentication_info(sip,value)            osip_message_set_header((osip_message_t *)sip,(const char *)"Authentication-Info",value)
+#endif
 /**
  * Get one Authentication-info header.
  * @param sip The element to work on.
  * @param pos The index of the element to get.
  * @param dest A pointer on the header found.
  */
-  int osip_message_get_authentication_info (const osip_message_t * sip, int pos,
-			       osip_authentication_info_t ** dest);
+#ifndef MINISIZE
+  int osip_message_get_authentication_info (const osip_message_t * sip,
+                                            int pos,
+                                            osip_authentication_info_t ** dest);
+#else
+#define osip_message_get_authentication_info(sip,pos,dest)          osip_message_header_get_byname(( osip_message_t *)sip,(const char *)"authentication-info",pos,(osip_header_t **)dest)
+#endif
 /**
  * Set the Authorization header.
  * @param sip The element to work on.
  * @param hvalue The string describing the element.
  */
-  int osip_message_set_authorization (osip_message_t * sip,
-				      const char *hvalue);
+  int osip_message_set_authorization (osip_message_t * sip, const char *hvalue);
 /**
  * Get one Authorization header.
  * @param sip The element to work on.
  * @param pos The index of the element to get.
  * @param dest A pointer on the header found.
  */
+#ifndef MINISIZE
   int osip_message_get_authorization (const osip_message_t * sip, int pos,
-				      osip_authorization_t ** dest);
+                                      osip_authorization_t ** dest);
+#else
+#define osip_message_get_authorization(sip, pos, dest) osip_message_get_knownheaderlist((&(sip)->authorizations), pos, (void **)(dest))
+#endif
 /**
  * Set the Call-id header.
  * @param sip The element to work on.
@@ -188,21 +239,33 @@ extern "C"
  * Get one Call-id header.
  * @param sip The element to work on.
  */
+#ifndef MINISIZE
   osip_call_id_t *osip_message_get_call_id (const osip_message_t * sip);
+#else
+#define osip_message_get_call_id(sip)   ((sip)->call_id)
+#endif
 /**
  * Set the Call-info header.
  * @param sip The element to work on.
  * @param hvalue The string describing the element.
  */
+#ifndef MINISIZE
   int osip_message_set_call_info (osip_message_t * sip, const char *hvalue);
+#else
+#define osip_message_set_call_info(sip,value)            osip_message_set_header((osip_message_t *)sip,(const char *)"Call-Info",value)
+#endif
 /**
  * Get one Call-info header.
  * @param sip The element to work on.
  * @param pos The index of the element to get.
  * @param dest A pointer on the header found.
  */
+#ifndef MINISIZE
   int osip_message_get_call_info (const osip_message_t * sip, int pos,
-				  osip_call_info_t ** dest);
+                                  osip_call_info_t ** dest);
+#else
+#define osip_message_get_call_info(sip,pos,dest)          osip_message_header_get_byname(( osip_message_t *)sip,(const char *)"call-info",pos,(osip_header_t **)dest)
+#endif
 /**
  * Set the Contact header.
  * @param sip The element to work on.
@@ -215,66 +278,65 @@ extern "C"
  * @param pos The index of the element to get.
  * @param dest A pointer on the header found.
  */
+#ifndef MINISIZE
   int osip_message_get_contact (const osip_message_t * sip, int pos,
-				osip_contact_t ** dest);
-/**
- * Set the Content-disposition header.
- * @param sip The element to work on.
- * @param hvalue The string describing the element.
- */
-  int osip_message_set_content_disposition (osip_message_t * sip,
-					    const char *hvalue);
-/**
- * Get one Content-disposition header.
- * @param sip The element to work on.
- * @param pos The index of the element to get.
- * @param dest A pointer on the header found.
- */
-  int osip_message_get_content_disposition (const osip_message_t * sip,
-					    int pos,
-					    osip_content_disposition_t **
-					    dest);
+                                osip_contact_t ** dest);
+#else
+#define osip_message_get_contact(sip, pos, dest) osip_message_get_knownheaderlist((&(sip)->contacts), pos, (void **)(dest)) 
+#endif
 /**
  * Set the Content-encoding header.
  * @param sip The element to work on.
  * @param hvalue The string describing the element.
  */
-  int osip_message_set_content_encoding (osip_message_t * sip,
-					 const char *hvalue);
+#ifndef MINISIZE
+  int osip_message_set_content_encoding (osip_message_t * sip, const char *hvalue);
+#else
+#define osip_message_set_content_encoding(sip,value)            osip_message_set_header((osip_message_t *)sip,(const char *)"Content-Encoding",value)
+#endif
 /**
  * Get one Content-encoding header.
  * @param sip The element to work on.
  * @param pos The index of the element to get.
  * @param dest A pointer on the header found.
  */
+#ifndef MINISIZE
   int osip_message_get_content_encoding (const osip_message_t * sip, int pos,
-					 osip_content_encoding_t ** dest);
+                                         osip_content_encoding_t ** dest);
+#else
+#define osip_message_get_content_encoding(sip,pos,dest)          osip_message_header_get_byname(( osip_message_t *)sip,(const char *)"content-encoding",pos,(osip_header_t **)dest)
+#endif
 /**
  * Set the Content-length header.
  * @param sip The element to work on.
  * @param hvalue The string describing the element.
  */
-  int osip_message_set_content_length (osip_message_t * sip,
-				       const char *hvalue);
+  int osip_message_set_content_length (osip_message_t * sip, const char *hvalue);
 /**
  * Get one Content-length header.
  * @param sip The element to work on.
  */
+#ifndef MINISIZE
   osip_content_length_t *osip_message_get_content_length (const osip_message_t
-							  * sip);
+                                                          * sip);
+#else
+#define osip_message_get_content_length(sip)   ((sip)->content_length)
+#endif
 /**
  * Set the Content-type header.
  * @param sip The element to work on.
  * @param hvalue The string describing the element.
  */
-  int osip_message_set_content_type (osip_message_t * sip,
-				     const char *hvalue);
+  int osip_message_set_content_type (osip_message_t * sip, const char *hvalue);
 /**
  * Get one Content-type header.
  * @param sip The element to work on.
  */
-  osip_content_type_t *osip_message_get_content_type (const osip_message_t *
-						      sip);
+#ifndef MINISIZE
+  osip_content_type_t *osip_message_get_content_type (const osip_message_t * sip);
+#else
+#define osip_message_get_content_type(sip)    ((sip)->content_type)
+#endif
 /**
  * Set the Cseq header.
  * @param sip The element to work on.
@@ -285,21 +347,33 @@ extern "C"
  * Get one Cseq header.
  * @param sip The element to work on.
  */
+#ifndef MINISIZE
   osip_cseq_t *osip_message_get_cseq (const osip_message_t * sip);
+#else
+#define osip_message_get_cseq(sip)  ((sip)->cseq)
+#endif
 /**
  * Set the Error-info header.
  * @param sip The element to work on.
  * @param hvalue The string describing the element.
  */
+#ifndef MINISIZE
   int osip_message_set_error_info (osip_message_t * sip, const char *hvalue);
+#else
+#define osip_message_set_error_info(sip,value)            osip_message_set_header((osip_message_t *)sip,(const char *)"Error-Info",value)
+#endif
 /**
  * Get one Error-info header.
  * @param sip The element to work on.
  * @param pos The index of the element to get.
  * @param dest A pointer on the header found.
  */
+#ifndef MINISIZE
   int osip_message_get_error_info (const osip_message_t * sip, int pos,
-				   osip_error_info_t ** dest);
+                                   osip_error_info_t ** dest);
+#else
+#define osip_message_get_error_info(sip,pos,dest)          osip_message_header_get_byname(( osip_message_t *)sip,(const char *)"error-info",pos,(osip_header_t **)dest)
+#endif
 /**
  * Set the From header.
  * @param sip The element to work on.
@@ -310,84 +384,109 @@ extern "C"
  * Get the From header.
  * @param sip The element to work on.
  */
+#ifndef MINISIZE
   osip_from_t *osip_message_get_from (const osip_message_t * sip);
+#else
+#define osip_message_get_from(sip)   ((sip)->from)
+#endif
 /**
  * Set the mime-version header.
  * @param sip The element to work on.
  * @param hvalue The string describing the element.
  */
-  int osip_message_set_mime_version (osip_message_t * sip,
-				     const char *hvalue);
+  int osip_message_set_mime_version (osip_message_t * sip, const char *hvalue);
 /**
  * Get the Mime-version header.
  * @param sip The element to work on.
  */
-  osip_mime_version_t *osip_message_get_mime_version (const osip_message_t *
-						      sip);
+#ifndef MINISIZE
+  osip_mime_version_t *osip_message_get_mime_version (const osip_message_t * sip);
+#else
+#define osip_message_get_mime_version(sip)   ((sip)->mime_version)
+#endif
 /**
  * Set the Proxy-authenticate header.
  * @param sip The element to work on.
  * @param hvalue The string describing the element.
  */
   int osip_message_set_proxy_authenticate (osip_message_t * sip,
-					   const char *hvalue);
+                                           const char *hvalue);
 /**
  * Get the Proxy-authenticate header.
  * @param sip The element to work on.
  * @param pos The index of the element to get.
  * @param dest A pointer on the header found.
  */
+#ifndef MINISIZE
   int osip_message_get_proxy_authenticate (const osip_message_t * sip,
-					   int pos,
-					   osip_proxy_authenticate_t ** dest);
+                                           int pos,
+                                           osip_proxy_authenticate_t ** dest);
+#else
+#define osip_message_get_proxy_authenticate(sip, pos, dest) osip_message_get_knownheaderlist((&(sip)->proxy_authenticates), pos, (void **)(dest)) 
+#endif
 /**
  * Set the Proxy-authorization header.
  * @param sip The element to work on.
  * @param hvalue The string describing the element.
  */
   int osip_message_set_proxy_authorization (osip_message_t * sip,
-					    const char *hvalue);
+                                            const char *hvalue);
 /**
  * Get one Proxy-authorization header.
  * @param sip The element to work on.
  * @param pos The index of the element to get.
  * @param dest A pointer on the header found.
  */
+#ifndef MINISIZE
   int osip_message_get_proxy_authorization (const osip_message_t * sip,
-					    int pos,
-					    osip_proxy_authorization_t **
-					    dest);
+                                            int pos,
+                                            osip_proxy_authorization_t ** dest);
+#else
+#define osip_message_get_proxy_authorization(sip, pos, dest) osip_message_get_knownheaderlist((&(sip)->proxy_authorizations), pos, (void **)(dest)) 
+#endif
 /**
  * Set the Proxy-authentication-info header.
  * @param sip The element to work on.
  * @param hvalue The string describing the element.
  */
+#ifndef MINISIZE
   int osip_message_set_proxy_authentication_info (osip_message_t * sip,
-					  const char *hvalue);
+                                                  const char *hvalue);
+#else
+#define osip_message_set_proxy_authentication_info(sip,value)            osip_message_set_header((osip_message_t *)sip,(const char *)"Proxy-Authentication-Info",value)
+#endif
 /**
  * Get the Proxy-authentication-info header.
  * @param sip The element to work on.
  * @param pos The index of the element to get.
  * @param dest A pointer on the header found.
  */
+#ifndef MINISIZE
   int osip_message_get_proxy_authentication_info (const osip_message_t * sip,
-				      int pos,
-				      osip_proxy_authentication_info_t ** dest);
+                                                  int pos,
+                                                  osip_proxy_authentication_info_t
+                                                  ** dest);
+#else
+#define osip_message_get_proxy_authentication_info(sip,pos,dest)          osip_message_header_get_byname(( osip_message_t *)sip,(const char *)"proxy-authentication-info",pos,(osip_header_t **)dest)
+#endif
 /**
  * Set the Record-Route header.
  * @param sip The element to work on.
  * @param hvalue The string describing the element.
  */
-  int osip_message_set_record_route (osip_message_t * sip,
-				     const char *hvalue);
+  int osip_message_set_record_route (osip_message_t * sip, const char *hvalue);
 /**
  * Get one Record-route header.
  * @param sip The element to work on.
  * @param pos The index of the element to get.
  * @param dest A pointer on the header found.
  */
+#ifndef MINISIZE
   int osip_message_get_record_route (const osip_message_t * sip, int pos,
-				     osip_record_route_t ** dest);
+                                     osip_record_route_t ** dest);
+#else
+#define osip_message_get_record_route(sip, pos, dest) osip_message_get_knownheaderlist((&(sip)->record_routes), pos, (void **)(dest)) 
+#endif
 /**
  * Set the Route header.
  * @param sip The element to work on.
@@ -400,8 +499,12 @@ extern "C"
  * @param pos The index of the element to get.
  * @param dest A pointer on the header found.
  */
+#ifndef MINISIZE
   int osip_message_get_route (const osip_message_t * sip, int pos,
-			      osip_route_t ** dest);
+                              osip_route_t ** dest);
+#else
+#define osip_message_get_route(sip, pos, dest) osip_message_get_knownheaderlist((&(sip)->routes), pos, (void **)(dest)) 
+#endif
 /**
  * Set the To header.
  * @param sip The element to work on.
@@ -412,7 +515,11 @@ extern "C"
  * Get the To header.
  * @param sip The element to work on.
  */
+#ifndef MINISIZE
   osip_to_t *osip_message_get_to (const osip_message_t * sip);
+#else
+#define osip_message_get_to(sip)    ((sip)->to)
+#endif
 /**
  * Set the Via header.
  * @param sip The element to work on.
@@ -431,23 +538,30 @@ extern "C"
  * @param pos The index of the element to get.
  * @param dest A pointer on the header found.
  */
+#ifndef MINISIZE
   int osip_message_get_via (const osip_message_t * sip, int pos,
-			    osip_via_t ** dest);
+                            osip_via_t ** dest);
+#else
+#define osip_message_get_via(sip, pos, dest) osip_message_get_knownheaderlist((&(sip)->vias), pos, (void **)(dest)) 
+#endif
 /**
  * Set the Www-authenticate header.
  * @param sip The element to work on.
  * @param hvalue The string describing the element.
  */
-  int osip_message_set_www_authenticate (osip_message_t * sip,
-					 const char *hvalue);
+  int osip_message_set_www_authenticate (osip_message_t * sip, const char *hvalue);
 /**
  * Get one Www-authenticate header.
  * @param sip The element to work on.
  * @param pos The index of the element to get.
  * @param dest A pointer on the header found.
  */
+#ifndef MINISIZE
   int osip_message_get_www_authenticate (const osip_message_t * sip, int pos,
-					 osip_www_authenticate_t ** dest);
+                                         osip_www_authenticate_t ** dest);
+#else
+#define osip_message_get_www_authenticate(sip, pos, dest) osip_message_get_knownheaderlist((&(sip)->www_authenticates), pos, (void **)(dest)) 
+#endif
 #ifndef DOXYGEN
 /**
  * Allocate and Add multiple header (not defined in oSIP).
@@ -456,7 +570,7 @@ extern "C"
  * @param hvalue The token value. VALUE MUST BE DYNAMICLY ALLOCATED
  */
   int osip_message_set_multiple_header (osip_message_t * sip, char *hname,
-					char *hvalue);
+                                        char *hvalue);
 #endif
 /**
  * Allocate and Add an "unknown" header (not defined in oSIP).
@@ -465,7 +579,8 @@ extern "C"
  * @param hvalue The token value.
  */
   int osip_message_set_header (osip_message_t * sip, const char *hname,
-			       const char *hvalue);
+                               const char *hvalue);
+#ifndef MINISIZE
 /**
  * Allocate and Add an "unknown" header (not defined in oSIP).
  * The element is add on the top of the unknown header list. 
@@ -474,8 +589,8 @@ extern "C"
  * @param hvalue The token value.
  */
   int osip_message_set_topheader (osip_message_t * sip, const char *hname,
-				  const char *hvalue);
-
+                                  const char *hvalue);
+#endif
 /**
  * Find an "unknown" header. (not defined in oSIP)
  * @param sip The element to work on.
@@ -484,17 +599,20 @@ extern "C"
  * @param dest A pointer to the header found.
  */
   int osip_message_header_get_byname (const osip_message_t * sip,
-				      const char *hname, int pos,
-				      osip_header_t ** dest);
+                                      const char *hname, int pos,
+                                      osip_header_t ** dest);
 /**
  * Get one "unknown" header.
  * @param sip The element to work on.
  * @param pos The index of the element to get.
  * @param dest A pointer on the header found.
  */
+#ifndef MINISZE
   int osip_message_get_header (const osip_message_t * sip, int pos,
-			       osip_header_t ** dest);
-
+                               osip_header_t ** dest);
+#else
+#define osip_message_get_header(sip, pos, dest) osip_message_get_knownheaderlist((&(sip)->headers), pos, (void **)(dest)) 
+#endif
 /**
  * Set the Body of the SIP message.
  * @param sip The element to work on.
@@ -508,20 +626,25 @@ extern "C"
  * @param buf the buffer containing the body.
  * @param length The length of the buffer.
  */
-  int osip_message_set_body_mime (osip_message_t * sip, const char *buf, size_t length);
+  int osip_message_set_body_mime (osip_message_t * sip, const char *buf,
+                                  size_t length);
 /**
  * Get one body header.
  * @param sip The element to work on.
  * @param pos The index of the element to get.
  * @param dest A pointer on the body found.
  */
+#ifndef MINISIZE
   int osip_message_get_body (const osip_message_t * sip, int pos,
-			     osip_body_t ** dest);
+                             osip_body_t ** dest);
+#else
+#define osip_message_get_body(sip, pos, dest) osip_message_get_knownheaderlist((&(sip)->bodies), pos, (void **)(dest)) 
+#endif
 
 
 
 /* trace facilities */
-#ifndef DOXYGEN			/* avoid DOXYGEN warning */
+#ifndef DOXYGEN                 /* avoid DOXYGEN warning */
 #ifdef ENABLE_TRACE
   void msg_logrequest (osip_message_t * sip, char *fmt);
   void msg_logresponse (osip_message_t * sip, char *fmt);
