@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001,2002,2003  Aymeric MOIZARD jack@atosc.org
+  Copyright (C) 2001,2002,2003,2004,2005,2006,2007 Aymeric MOIZARD jack@atosc.org
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -51,6 +51,7 @@ void
 osip_mutex_destroy (struct osip_mutex *_mut)
 {
   osip_mutex_t *mut = (osip_mutex_t *) _mut;
+
   if (mut == NULL)
     return;
   pthread_mutex_destroy (mut);
@@ -61,6 +62,7 @@ int
 osip_mutex_lock (struct osip_mutex *_mut)
 {
   osip_mutex_t *mut = (osip_mutex_t *) _mut;
+
   if (mut == NULL)
     return -1;
   return pthread_mutex_lock (mut);
@@ -70,6 +72,7 @@ int
 osip_mutex_unlock (struct osip_mutex *_mut)
 {
   osip_mutex_t *mut = (osip_mutex_t *) _mut;
+
   if (mut == NULL)
     return -1;
   return pthread_mutex_unlock (mut);
@@ -77,13 +80,100 @@ osip_mutex_unlock (struct osip_mutex *_mut)
 
 #endif
 
-#if (defined(HAVE_SEMAPHORE_H) && !defined(__APPLE_CC__)) || defined(HAVE_PTHREAD_WIN32)
+#if defined(__arc__)
 
 /* Counting Semaphore is initialized to value */
 struct osip_sem *
 osip_sem_init (unsigned int value)
 {
   osip_sem_t *sem = (osip_sem_t *) osip_malloc (sizeof (osip_sem_t));
+
+  if (sem == NULL)
+    return NULL;
+
+  sem->_sem_counter=0;
+  sem->_sem_mutex=osip_mutex_init ();
+  if (sem->_sem_mutex!=NULL)
+    return (struct osip_sem *) sem;
+  osip_free (sem);
+  return NULL;
+}
+
+int
+osip_sem_destroy (struct osip_sem *_sem)
+{
+  osip_sem_t *sem = (osip_sem_t *) _sem;
+
+  if (sem == NULL)
+    return 0;
+  osip_mutex_destroy(sem->_sem_mutex);
+  osip_free (sem);
+  return 0;
+}
+
+int
+osip_sem_post (struct osip_sem *_sem)
+{
+  osip_sem_t *sem = (osip_sem_t *) _sem;
+
+  if (sem == NULL)
+    return -1;
+  osip_mutex_lock(sem->_sem_mutex);
+  sem->_sem_counter++;
+  osip_mutex_unlock(sem->_sem_mutex);
+  return 0;
+}
+
+int
+osip_sem_wait (struct osip_sem *_sem)
+{
+  osip_sem_t *sem = (osip_sem_t *) _sem;
+
+  if (sem == NULL)
+    return -1;
+
+  /* poor emulation... */
+  while (1)
+    {
+      osip_mutex_lock(sem->_sem_mutex);
+      if (sem->_sem_counter>0)
+	{
+	  sem->_sem_counter--;
+	  osip_mutex_unlock(sem->_sem_mutex);
+	  return 0;
+	}
+      osip_mutex_unlock(sem->_sem_mutex);
+      osip_usleep(1000);
+    }
+  return -1;
+}
+
+int
+osip_sem_trywait (struct osip_sem *_sem)
+{
+  osip_sem_t *sem = (osip_sem_t *) _sem;
+
+  if (sem == NULL)
+    return -1;
+  osip_mutex_lock(sem->_sem_mutex);
+  if (sem->_sem_counter>0)
+    {
+      sem->_sem_counter--;
+      osip_mutex_unlock(sem->_sem_mutex);
+      return 0;
+    }
+  osip_mutex_unlock(sem->_sem_mutex);
+  return -1;
+}
+
+#elif (defined(HAVE_SEMAPHORE_H) && !defined(__APPLE_CC__)) || defined(HAVE_PTHREAD_WIN32)
+
+/* Counting Semaphore is initialized to value */
+struct osip_sem *
+osip_sem_init (unsigned int value)
+{
+  osip_sem_t *sem = (osip_sem_t *) osip_malloc (sizeof (osip_sem_t));
+
   if (sem == NULL)
     return NULL;
 
@@ -97,6 +187,7 @@ int
 osip_sem_destroy (struct osip_sem *_sem)
 {
   osip_sem_t *sem = (osip_sem_t *) _sem;
+
   if (sem == NULL)
     return 0;
   sem_destroy (sem);
@@ -108,6 +199,7 @@ int
 osip_sem_post (struct osip_sem *_sem)
 {
   osip_sem_t *sem = (osip_sem_t *) _sem;
+
   if (sem == NULL)
     return -1;
   return sem_post (sem);
@@ -117,6 +209,7 @@ int
 osip_sem_wait (struct osip_sem *_sem)
 {
   osip_sem_t *sem = (osip_sem_t *) _sem;
+
   if (sem == NULL)
     return -1;
   return sem_wait (sem);
@@ -126,6 +219,7 @@ int
 osip_sem_trywait (struct osip_sem *_sem)
 {
   osip_sem_t *sem = (osip_sem_t *) _sem;
+
   if (sem == NULL)
     return -1;
   return sem_trywait (sem);
@@ -169,6 +263,7 @@ osip_sem_destroy (struct osip_sem *_sem)
 {
   union semun val;
   osip_sem_t *sem = (osip_sem_t *) _sem;
+
   if (sem == NULL)
     return 0;
   val.val = 0;
@@ -228,13 +323,14 @@ osip_sem_trywait (struct osip_sem *_sem)
 struct osip_mutex *
 osip_mutex_init ()
 {
-  return (struct osip_mutex *) semMCreate (SEM_Q_FIFO|SEM_DELETE_SAFE);
+  return (struct osip_mutex *) semMCreate (SEM_Q_FIFO | SEM_DELETE_SAFE);
 }
 
 void
 osip_mutex_destroy (struct osip_mutex *_mut)
 {
   osip_mutex_t *mut = (osip_mutex_t *) _mut;
+
   if (mut == NULL)
     return;
   semDelete (mut);
@@ -244,6 +340,7 @@ int
 osip_mutex_lock (struct osip_mutex *_mut)
 {
   osip_mutex_t *mut = (osip_mutex_t *) _mut;
+
   if (mut == NULL)
     return -1;
   return semTake (mut, WAIT_FOREVER);
@@ -253,6 +350,7 @@ int
 osip_mutex_unlock (struct osip_mutex *_mut)
 {
   osip_mutex_t *mut = (osip_mutex_t *) _mut;
+
   if (mut == NULL)
     return -1;
   return semGive (mut);
@@ -278,6 +376,7 @@ int
 osip_sem_destroy (struct osip_sem *_sem)
 {
   osip_sem_t *sem = (osip_sem_t *) _sem;
+
   if (sem == NULL)
     return 0;
   semDelete (sem->semId);
@@ -289,6 +388,7 @@ int
 osip_sem_post (struct osip_sem *_sem)
 {
   osip_sem_t *sem = (osip_sem_t *) _sem;
+
   if (sem == NULL)
     return -1;
   return semGive (sem->semId);
@@ -298,6 +398,7 @@ int
 osip_sem_wait (struct osip_sem *_sem)
 {
   osip_sem_t *sem = (osip_sem_t *) _sem;
+
   if (sem == NULL)
     return -1;
   return semTake (sem->semId, WAIT_FOREVER);
@@ -307,6 +408,7 @@ int
 osip_sem_trywait (struct osip_sem *_sem)
 {
   osip_sem_t *sem = (osip_sem_t *) _sem;
+
   if (sem == NULL)
     return -1;
   return semTake (sem->semId, NO_WAIT);
@@ -316,13 +418,63 @@ osip_sem_trywait (struct osip_sem *_sem)
 
 
 #if (defined(WIN32) || defined(_WIN32_WCE)) && !defined(HAVE_PTHREAD_WIN32)
-
 #include <limits.h>
 
+#if (_WIN32_WINNT >= 0x0500)
+struct osip_mutex *
+osip_mutex_init ()
+{
+  osip_mutex_t *mut = (osip_mutex_t *) osip_malloc (sizeof (osip_mutex_t));
+
+  if (mut == NULL)
+    return NULL;
+  if (InitializeCriticalSectionAndSpinCount
+      (&mut->h, OSIP_CRITICALSECTION_SPIN) != 0)
+    return (struct osip_mutex *) (mut);
+  osip_free (mut);
+  return (NULL);
+}
+
+void
+osip_mutex_destroy (struct osip_mutex *_mut)
+{
+  osip_mutex_t *mut = (osip_mutex_t *) _mut;
+
+  if (mut == NULL)
+    return;
+  DeleteCriticalSection (&mut->h);
+  osip_free (mut);
+}
+
+int
+osip_mutex_lock (struct osip_mutex *_mut)
+{
+  osip_mutex_t *mut = (osip_mutex_t *) _mut;
+
+  if (mut == NULL)
+    return -1;
+  EnterCriticalSection (&mut->h);
+
+  return (0);
+}
+
+int
+osip_mutex_unlock (struct osip_mutex *_mut)
+{
+  osip_mutex_t *mut = (osip_mutex_t *) _mut;
+
+  if (mut == NULL)
+    return -1;
+  LeaveCriticalSection (&mut->h);
+  return (0);
+}
+#else
+
 struct osip_mutex *
 osip_mutex_init ()
 {
   osip_mutex_t *mut = (osip_mutex_t *) osip_malloc (sizeof (osip_mutex_t));
+
   if (mut == NULL)
     return NULL;
   if ((mut->h = CreateMutex (NULL, FALSE, NULL)) != NULL)
@@ -335,6 +487,7 @@ void
 osip_mutex_destroy (struct osip_mutex *_mut)
 {
   osip_mutex_t *mut = (osip_mutex_t *) _mut;
+
   if (mut == NULL)
     return;
   CloseHandle (mut->h);
@@ -358,16 +511,19 @@ int
 osip_mutex_unlock (struct osip_mutex *_mut)
 {
   osip_mutex_t *mut = (osip_mutex_t *) _mut;
+
   if (mut == NULL)
     return -1;
   ReleaseMutex (mut->h);
   return (0);
 }
+#endif
 
 struct osip_sem *
 osip_sem_init (unsigned int value)
 {
   osip_sem_t *sem = (osip_sem_t *) osip_malloc (sizeof (osip_sem_t));
+
   if (sem == NULL)
     return NULL;
 
@@ -381,6 +537,7 @@ int
 osip_sem_destroy (struct osip_sem *_sem)
 {
   osip_sem_t *sem = (osip_sem_t *) _sem;
+
   if (sem == NULL)
     return 0;
   CloseHandle (sem->h);
@@ -392,6 +549,7 @@ int
 osip_sem_post (struct osip_sem *_sem)
 {
   osip_sem_t *sem = (osip_sem_t *) _sem;
+
   if (sem == NULL)
     return -1;
   ReleaseSemaphore (sem->h, 1, NULL);
@@ -443,6 +601,7 @@ void
 osip_mutex_destroy (struct osip_mutex *_mut)
 {
   osip_mutex_t *mut = (osip_mutex_t *) _mut;
+
   if (mut)
     {
       sm_delete (mut->id);
@@ -454,10 +613,11 @@ int
 osip_mutex_lock (struct osip_mutex *_mut)
 {
   osip_mutex_t *mut = (osip_mutex_t *) _mut;
+
   if (mut)
     {
       if (sm_p (mut->id, SM_WAIT, 0) != 0)
-	return (-1);
+        return (-1);
     }
   return (0);
 }
@@ -466,6 +626,7 @@ int
 osip_mutex_unlock (struct osip_mutex *_mut)
 {
   osip_mutex_t *mut = (osip_mutex_t *) _mut;
+
   if (mut)
     {
       sm_v (mut->id);
@@ -489,6 +650,7 @@ int
 osip_sem_destroy (struct osip_sem *_sem)
 {
   osip_sem_t *sem = (osip_sem_t *) _sem;
+
   if (sem == NULL)
     return 0;
   sm_delete (sem->id);
@@ -500,6 +662,7 @@ int
 osip_sem_post (struct osip_sem *_sem)
 {
   osip_sem_t *sem = (osip_sem_t *) _sem;
+
   if (sem == NULL)
     return -1;
   return (sm_v (sem->id));
@@ -509,6 +672,7 @@ int
 osip_sem_wait (struct osip_sem *_sem)
 {
   osip_sem_t *sem = (osip_sem_t *) _sem;
+
   if (sem == NULL)
     return -1;
   if (sm_p (sem->id, SM_WAIT, 0) != 0)
@@ -520,6 +684,7 @@ int
 osip_sem_trywait (struct osip_sem *_sem)
 {
   osip_sem_t *sem = (osip_sem_t *) _sem;
+
   if (sem == NULL)
     return -1;
   if (sm_p (sem->id, SM_NOWAIT, 0) != 0)
