@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001,2002,2003  Aymeric MOIZARD jack@atosc.org
+  Copyright (C) 2001,2002,2003,2004,2005,2006,2007 Aymeric MOIZARD jack@atosc.org
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -26,7 +26,6 @@
 
 #if (defined(WIN32) || defined(_WIN32_WCE)) && !defined(HAVE_PTHREAD_WIN32)
 #  if defined _WIN32_WCE
-#    include <winbase.h>
 #    define _beginthreadex	CreateThread
 #    define	_endthreadex	ExitThread
 #  elif defined WIN32
@@ -78,7 +77,9 @@ osip_thread_join (struct osip_thread *_thread)
 void
 osip_thread_exit ()
 {
+#if !defined(__arc__)
   pthread_exit (NULL);
+#endif
 }
 
 #endif
