@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001,2002,2003,2004,2005  Aymeric MOIZARD jack@atosc.org
+  Copyright (C) 2001,2002,2003,2004,2005,2006,2007 Aymeric MOIZARD jack@atosc.org
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -70,6 +70,7 @@ osip_message_set_header (osip_message_t * sip, const char *hname,
   return 0;                     /* ok */
 }
 
+#ifndef MINISIZE
 /* Add a header to a SIP message at the top of the list.    */
 /* INPUT :  char *hname | pointer to a header name.         */
 /* INPUT :  char *hvalue | pointer to a header value.       */
@@ -126,8 +127,9 @@ osip_message_get_header (const osip_message_t * sip, int pos,
   if (osip_list_size (&sip->headers) <= pos)
     return -1;                  /* NULL */
   *dest = (osip_header_t *) osip_list_get (&sip->headers, pos);
-  return 0;
+  return pos;
 }
+#endif
 
 /* Get a header in a SIP message.                       */
 /* INPUT : int pos | position where we start the search */
