@@ -1,6 +1,6 @@
 /*
   The oSIP library implements the Session Initiation Protocol (SIP -rfc3261-)
-  Copyright (C) 2001,2002,2003,2004,2005  Aymeric MOIZARD jack@atosc.org
+  Copyright (C) 2001,2002,2003,2004,2005,2006,2007 Aymeric MOIZARD jack@atosc.org
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -22,6 +22,7 @@
 #include <windows.h>
 #endif
 
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <stdarg.h>
@@ -81,7 +82,7 @@
 #  include <unistd.h>
 #endif
 
-#if defined (HAVE_SYSLOG_H)
+#if defined (HAVE_SYSLOG_H) && !defined(__arc__)
 #  include <syslog.h>
 #endif
 
@@ -93,6 +94,9 @@
 #include <pthread.h>
 #endif
 
+#if defined(__arc__)
+#define HAVE_LRAND48
+#endif
 
 FILE *logfile = NULL;
 int tracing_table[END_TRACE_LEVEL];
@@ -101,11 +105,13 @@ static osip_trace_func_t *trace_func = 0;
 
 static unsigned int random_seed_set = 0;
 
-#ifndef WIN32
+#ifndef MINISIZE
+#if !defined(WIN32) && !defined(_WIN32_WCE)
 osip_malloc_func_t *osip_malloc_func = 0;
 osip_realloc_func_t *osip_realloc_func = 0;
 osip_free_func_t *osip_free_func = 0;
 #endif
+#endif
 
 #ifndef WIN32_USE_CRYPTO
 unsigned int
@@ -171,7 +177,20 @@ osip_fallback_random_number ()
       random_seed_set = 1;
     }
 #ifdef HAVE_LRAND48
-  return lrand48 ();
+  {
+    int val = lrand48();
+    if (val==0)
+      {
+	unsigned int ticks;
+	struct timeval tv;
+	gettimeofday (&tv, NULL);
+	ticks = tv.tv_sec + tv.tv_usec;
+	srand48 (ticks);
+	return lrand48 ();
+      }
+
+    return val;
+  }
 #else
   return rand ();
 #endif
@@ -206,6 +225,32 @@ osip_build_random_number ()
 #include <limits.h>
 #endif
 
+
+char *
+osip_strncpy (char *dest, const char *src, size_t length)
+{
+  strncpy (dest, src, length);
+  dest[length] = '\0';
+  return dest;
+}
+
+#undef osip_strdup
+
+char *
+osip_strdup (const char *ch)
+{
+  char *copy;
+  size_t length;
+
+  if (ch == NULL)
+    return NULL;
+  length = strlen (ch);
+  copy = (char *) osip_malloc (length + 1);
+  osip_strncpy (copy, ch, length);
+  return copy;
+}
+
+#ifndef MINISIZE
 int
 osip_atoi (const char *number)
 {
@@ -223,13 +268,7 @@ osip_atoi (const char *number)
   return atoi (number);
 }
 
-char *
-osip_strncpy (char *dest, const char *src, size_t length)
-{
-  strncpy (dest, src, length);
-  dest[length] = '\0';
-  return dest;
-}
+#endif
 
 /* append string_osip_to_append to string at position cur
    size is the current allocated size of the element
@@ -268,6 +307,12 @@ osip_usleep (int useconds)
     ;
 #elif defined(WIN32)
   Sleep (useconds / 1000);
+#elif defined(__arc__)
+  struct timespec req;
+  struct timespec rem;
+  req.tv_sec = (int) useconds / 1000000;
+  req.tv_nsec = (int) (useconds % 1000000)*1000;
+  nanosleep (&req, &rem);
 #else
   struct timeval delay;
   int sec;
@@ -286,22 +331,6 @@ osip_usleep (int useconds)
 #endif
 }
 
-#undef osip_strdup
-
-char *
-osip_strdup (const char *ch)
-{
-  char *copy;
-  size_t length;
-
-  if (ch == NULL)
-    return NULL;
-  length = strlen (ch);
-  copy = (char *) osip_malloc (length + 1);
-  osip_strncpy (copy, ch, length);
-  return copy;
-}
-
 char *
 osip_strdup_without_quote (const char *ch)
 {
@@ -320,7 +349,8 @@ osip_strdup_without_quote (const char *ch)
 int
 osip_tolower (char *word)
 {
-#ifdef HAVE_CTYPE_H
+#if defined(HAVE_CTYPE_H) && !defined (_WIN32_WCE)
+
   for (; *word; word++)
     *word = (char) tolower (*word);
 #else
@@ -336,6 +366,7 @@ osip_tolower (char *word)
   return 0;
 }
 
+#ifndef MINISIZE
 int
 osip_strcasecmp (const char *s1, const char *s2)
 {
@@ -378,6 +409,7 @@ osip_strncasecmp (const char *s1, const char *s2, size_t len)
   return _strnicmp (s1, s2, len);
 #endif
 }
+#endif
 
 /* remove SPACE before and after the content */
 int
@@ -677,7 +709,7 @@ osip_trace_initialize_syslog (osip_trace_level_t level, char *ident)
 {
   osip_trace_level_t i = 0;
 
-#if defined (HAVE_SYSLOG_H)
+#if defined (HAVE_SYSLOG_H) && !defined(__arc__)
   openlog (ident, LOG_CONS | LOG_PID, LOG_DAEMON);
   use_syslog = 1;
 #endif
@@ -799,7 +831,7 @@ osip_trace (char *fi, int li, osip_trace_level_t level, FILE * f, char *chfr, ..
     {
       trace_func (fi, li, level, chfr, ap);
     }
-#if defined (HAVE_SYSLOG_H)
+#if defined (HAVE_SYSLOG_H) && !defined(__arc__)
   else if (use_syslog == 1)
     {
       char buffer[512];
@@ -878,7 +910,7 @@ osip_trace (char *fi, int li, osip_trace_level_t level, FILE * f, char *chfr, ..
 
 
 
-#ifdef WIN32
+#if defined(WIN32) || defined(_WIN32_WCE)
 
 #undef osip_malloc
 void *
@@ -909,6 +941,7 @@ osip_free (void *ptr)
 
 #else
 
+#ifndef MINISIZE
 void
 osip_set_allocators (osip_malloc_func_t * malloc_func,
                      osip_realloc_func_t * realloc_func,
@@ -918,6 +951,7 @@ osip_set_allocators (osip_malloc_func_t * malloc_func,
   osip_realloc_func = realloc_func;
   osip_free_func = free_func;
 }
+#endif
 
 #endif
 
@@ -1085,13 +1119,13 @@ _osip_realloc (void *ptr, size_t size, char *file, unsigned short line)
 unsigned long
 osip_hash (const char *str)
 {
-  unsigned long hash = 5381;
+  unsigned int hash = 5381;
   int c;
 
   while (c = *str++)
     hash = ((hash << 5) + hash) + c;
 
-  return hash;
+  return hash & 0xFFFFFFFFu;
 }
 
 /* ---For better performance---
@@ -1136,6 +1170,7 @@ osip_strn_append (char *dst, const char *src, size_t len)
    This is to replace this construction:
    osip_strncpy (  dest, source, length);
    osip_clrspace ( dest ); */
+#ifndef MINISIZE
 char *
 osip_clrncpy (char *dst, const char *src, size_t len)
 {
@@ -1181,3 +1216,15 @@ osip_clrncpy (char *dst, const char *src, size_t len)
 
   return dst;
 }
+
+#else
+
+char *
+osip_clrncpy (char *dst, const char *src, size_t len)
+{
+   osip_strncpy ( dst, src, len);
+   osip_clrspace ( dst );
+   return dst;
+}
+
+#endif
